/**
 * 
 */
package sim.application.util;

import java.io.IOException;

import sim.graphics.SScene;
import sim.readwrite.SKeyWordDecoder;
import sim.util.SLog;
import sim.util.SStringUtil;

/**
 * La classe <b>SRendererInfo</b> reprsente un gnrateur d'information dcrivant un mode d'usage de l'application SIMRenderer.
 * Cette classe gnre un text permettant d'claicir le mode de fonctionnement des fichiers suivants :
 * <ul>- Description gnrale de l'application</ul>
 * <ul>- Objectif de l'application</ul>
 * <ul>- Le fichier configuration.cfg</ul>
 * <ul>- Le fichier log.txt</ul>
 * <ul>- La structure d'une fichier de scene (ex : scene.txt)</ul>
 * 
 * @author Simon Vzina
 * @since 2016-02-28
 * @version 2016-05-20
 */
public class SRendererInfo {

  //--------------
  // CONSTANTES //
  //--------------
  
  /**
   * La constante <b>NB_KEY_WORD_PER_LINE</b> correspond au nombre de mots cls qui seront crit par ligne de texte tant gal  {@value}.
   */
  private static int NB_KEY_WORD_PER_LINE = 5;
  
  //----------------
  // CONSTRUCTEUR //
  //----------------
  
  /**
   * Constructeur d'un gnrateur d'information sur l'application SIMRenderer. 
   */
  public SRendererInfo()
  {
    
  }

  //------------
  // MTHODES //
  //------------
  
  /**
   * Mthode pour faire l'criture de la scne par dfaut.
   */
  public void writeDefaultScene()
  {
    try{   
      // Chargement de la scne par dfaut
      SScene scene = new SScene(SScene.DEFAULT_SCENE_FILE_NAME); 
      
      SLog.logWriteLine("Message SRendererInfo : criture du fichier de scne '" + SScene.DEFAULT_SCENE_FILE_NAME + "'.");
      
      // crire un fichier de scne avec le nom par dfaut
      scene.write(SScene.DEFAULT_SCENE_FILE_NAME);                
     
    }catch(IOException e){
      // rien  faire ...
    }
  }
  
  /**
   * Mthode pour faire l'affichage d'un message d'information en lien avec l'application SIMRenderer.
   */
  public void writeInfo()
  {
    SLog.logWriteLine("Bienvenu dans l'application 'info' de SIMRenderer.");
    SLog.logWriteLine();
    
    SLog.logWriteLine("Description de l'application SIMRenderer :");
    SLog.logWriteLine("Cette application est un gnrateur d'image de synthse  partir d'une scne dcrite en 3D  l'aide d'un algorithme de ray tracing.");
    SLog.logWriteLine("Un ray tracer permet de lancer des rayons depuis la position d'une camra dans l'ensemble des pixels d'un cran situ dans la scne.");
    SLog.logWriteLine(" partir de calcul d'intersection entre le rayon et les gomtries de la scne, on peut valuer la visibilit de ceux-ci.");
    SLog.logWriteLine("La couleur attribue  tous les pixels dpend d'un algorithme d'illumination pseudo-physique.");
    SLog.logWriteLine("En attribuant des matriaux aux gomtries et en spcifiant l'clairage de la scne, on peut gnrer une image 2D de grande qualit.");
    SLog.logWriteLine();
    
    SLog.logWriteLine("Objectif de l'application SIMRenderer :");
    SLog.logWriteLine("L'application est avant tout une plate-forme d'apprentissage en lien avec les mathmatiques, la physique et l'informatique.");
    SLog.logWriteLine("Ce programme n'est pas d'intrt commercial ni industriel.");
    SLog.logWriteLine("Pour toute question  ce sujet, veuillez contacter Simon Vzina (svezina@cmaisonneuve.qc.ca), l'auteur de ce programme et de ce projet ducatif."); 
    SLog.logWriteLine();
    
    SLog.logWriteLine();
    SLog.logWriteLine();
    
    SLog.logWriteLine("Information sur le fichier 'configuration.cfg' :");
    SLog.logWriteLine("Dans la variable 'application', vous pouvez choisir parmi les applications suivantes : " + SStringUtil.join(SConfiguration.APPLICATION_TYPE, ", "));
    SLog.logWriteLine("Dans la variable 'read_data', vous devez prciser le nom du fichier de scne que vous voulez charger. (ex : scene.txt)");
    SLog.logWriteLine();
    
    SLog.logWriteLine("Information sur le fichier 'log.txt' :");
    SLog.logWriteLine("Ce fichier contient des informations dcrivant le fonctionnement de la dernire excution du programme.");
    SLog.logWriteLine("Ce fichier doit tre consult si une erreur est survenue et que l'on cherche de l'information  ce sujet.");
    SLog.logWriteLine();
    
    SLog.logWriteLine("Information sur les fichiers de scne :");
    SLog.logWriteLine("Ce type de fichier contient la description complte de la scne 3D.");
    SLog.logWriteLine("Il reconnait un ensemble de " + SKeyWordDecoder.getNbKeyWord() + " mots cls pouvant tre utiliss pour construire ou dfinir des objets.");
    SLog.logWriteLine("Certains de ces mots cls se retrouvent en version 'franaise' et 'anglaise'.");
    SLog.logWriteLine("Pour modifier un fichier de scne, il suffit d'crire un 'mot cl' pour construire un objet et de dfinir ses attribues avec d'autres mots cl.");
    SLog.logWriteLine("Une erreur de lecture engendra un message d'erreur prcisant la nature de l'erreur et proposera des mots cls valides.");
    SLog.logWriteLine();
    
    SLog.logWriteLine("Liste des mots cls reconnus par le systme :");
    writeKeyWord();
    
    SLog.logWriteLine("Fin de l'application info.");
    SLog.logWriteLine();
  }
  
  /**
   * Mthode pour faire l'criture des mots cls reconnus par la classe SKeyWordDecoder.
   */
  private static void writeKeyWord()
  {
    String[] tab = SKeyWordDecoder.getKeyWord();
    
    int j = 0;
    
    for(int i = 0; i < tab.length; i++)
    {
      SLog.logWrite(tab[i] + ", ");
      j++;
      
      // crire 
      if(j > NB_KEY_WORD_PER_LINE)
      {
        SLog.logWriteLine();
        j = 0;
      }
    }
    SLog.logWriteLine();
    SLog.logWriteLine();
  }
  
}//fin de la classe SRendererInfo
