/**
 * 
 */
package sim.geometry;

import sim.exception.SRuntimeException;
import sim.graphics.SPrimitive;
import sim.math.SVector3d;
import sim.readwrite.SWriteable;

/**
 * L'interface <b>SGeometry</b> reprsentant une gomtrie. 
 * Une gomtrie doit : 
 * <ul>tre intersectable par une rayon, </ul> 
 * <ul>dfinir une surface ferme ou non, </ul>
 * <ul>et identifier si un vecteur position se trouve  l'intrieur de celle-ci (si elle est ferme). </ul>
 * @author Simon Vzina
 * @since 2015-01-10
 * @version 2017-08-22
 */
public interface SGeometry extends SWriteable {

	/**
	 * Mthode pour obtenir le numro d'identification unique ID de la gomtrie.
	 * @return Le numro d'identification ID.
	 */
	public long getID();
	
	/**
	 * Mthode pour obtenir le numro correspondant au nom de la gomtrie.
	 * @return Le numro correspondant au nom de la gomtrie.
	 */
	public int getCodeName();
	
	/**
   * Mthode pour obtenir la primitive parent dont cette gomtrie fait partie.
   * @return La primitive parent de cette gomtrie.
   * @throws SRuntimeException Si la gomtrie ne possde pas de primitive comme parent.
   */
  public SPrimitive getPrimitiveParent()throws SRuntimeException;
  
  /**
   * Mthode pour affecter primitive commet parent  la gomtrie. 
   * @param parent - La primitive parent de la gomtrie.
   * @throws SRuntimeException Si la gomtrie possde dj un parent (n'est pas pralablement 'null').
   */
  public void setPrimitiveParent(SPrimitive parent)throws SRuntimeException;
  
	/**
	 * Mthode qui dtermine si la gomtrie est transparente.
	 * @return <b> true </b> si la gomtrie est transparente et <b> false </b> sinon.
	 */
	public boolean isTransparent();
	
	/**
	 * Mthode qui dtermine si la gomtrie est une surface ferme. Ainsi, on peut y dfinir l'intrieur et l'extrieur. 
	 * @return <b> true </b> si la gomtrie est ferme et <b> false </b> sinon.
	 */
	public boolean isClosedGeometry();
	
	/**
	 * Mthode qui dtermine si un vecteur point  l'intrieur de la gomtrie. Si la gomtrie n'est pas ferme,
	 * le vecteur point automatiquement  l'extrieur de la gomtrie.
	 * @param v - Le vecteur.
	 * @return <b> true </b> si le vecteur point  l'intrieur de la gomtrie et <b> false </b> sinon.
	 */
	public boolean isInside(SVector3d v);
	
	/**
   * Mthode permettant d'effectuer le calcul de l'intersection (s'il y a) entre un rayon et une gomtrie.
   * @param ray - Le rayon  intersecter avec la gomtrie.
   * @return Un rayon intersect avec les caractristiques de l'intersection s'il y a eu intersection ou le rayon pass en paramtre s'il n'y a pas eu d'intersection.
   * @throws SAlreadyIntersectedRayException Si la gomtrie intersecte un rayon tant dj dans un tant intersect.
   */
  public SRay intersection(SRay ray) throws SAlreadyIntersectedRayException;
	
}//fin interface SGeometry
