/**
 * 
 */
package sim.graphics;

import java.util.HashMap;
import java.util.List;
import java.util.Map;

import sim.graphics.material.SDefaultMaterial;
import sim.graphics.material.SMaterial;
import sim.util.SLog;

/**
 * Classe qui permet d'affecter le bon matriel  la bonne primitive en fonction du nom du matriel demand par la primitive.
 *
 * @author Simon Vzina
 * @since 2015-07-31
 * @version 2015-07-31
 */
public class SMaterialAffectation {

  private final List<SPrimitive> primitive_list;  //liste des primitives que l'on doit affecter un matriel
  private final List<SMaterial> material_list;    //liste des matriaux disponibles pour l'affectation
  
  /**
   * Constructeur de l'assignateur de matriel aux primitives.
   * @param primitive_list - La liste des primitives qui doivent subir une affectation d'un matriel.
   * @param material_list - La liste des matriaux disponibles pour l'affectation.
   */
  public SMaterialAffectation(List<SPrimitive> primitive_list, List<SMaterial> material_list)
  {
    this.primitive_list = primitive_list;
    this.material_list = material_list;
  }
  
  /**
   * Mthode pour raliser l'affectation des matriaux aux primitives. Si un matriau est demand (par son nom tant la cl d'identification)
   * et qu'il n'est pas prsent dans la liste, il sera cr avec des paramtres par dfaut. Un message sera affich pour signaler cette situation.
   */
  public void affectation()
  {
    //Remplir une carte de matriel avec le nom du matriel en cl d'accs.
    //Un message sera envoy si deux matriaux ont le mme nom.
    Map<String, SMaterial> material_map = new HashMap<String, SMaterial>();
    
    for(SMaterial m : material_list)
      if(material_map.containsKey(m.getName()))
        SLog.logWriteLine("Message SMaterialAffectation : Le nom de matriel '" + m.getName() + "' se retrouve deux fois au plus dans la scne. Seule la premire dfinition sera affecte aux primitives.");
      else
        material_map.put(m.getName(), m);
      
    
    //Affecter les matriels aux primitives qui n'ont pas de matriel (ceux provenant d'un modle devrait avoir un matriel pralablement affect)
    //Affecter un matriel  la primitive si le nom du matriel demand par la primitive se retrouve dans la map.
    //Cration d'un matriel par dfaut avec le nom demand par la primitive si une primitive n'a pas de matriel (pour qu'elle soit visible puisqu'elle a une gomtrie).
    for(SPrimitive p : primitive_list)
      if(material_map.containsKey(p.getMaterialName()))        //vrifier si le nom du matriel dsir est pralablement disponible
        p.setMaterial(material_map.get(p.getMaterialName()));
      else                                                     
      {
        //Construire un nouveau matriel, car celui recherch n'est pas disponible !
        SLog.logWriteLine("Message SMaterialAffectation : Le matriel '" + p.getMaterialName() + "' demand par une primitive ne se retrouve pas dans la collection des matriaux. Un nouveau matriel sera cr par dfaut.");
          
        //Cration d'un nouveau matriel de couleur blanche avec le nom dsir par la primitive
        SMaterial new_material = new SDefaultMaterial(p.getMaterialName()); 
          
        //Ajout  la liste des matriels (ncessaire pour la sauvegarde de la scne)
        material_list.add(new_material);
          
        //Ajout de ce matriel  la carte temporaire (au cas o il y aurait d'autres primitives qui voudraient utiliser ce matriel).
        material_map.put(new_material.getName(), new_material);
          
        p.setMaterial(new_material);  //Affectation du nouveau matriel  la primitive
      }
  }
  
}//fin de la classe SMaterialAffectation
