/**
 * 
 */
package sim.graphics.light;

import sim.math.SVector3d;

/**
 * L'interface <b>SInterferenceLight</b> reprsente une source de lumire dont l'intensit de la source
 * dpend de l'interfrence interne de la source.
 *  
 * @author Simon Vzina
 * @since 2016-02-15
 * @version 2016-02-28
 */
public interface SInterferenceLight extends SAttenuatedLight {

  /**
   * <p>
   * Mthode permettant d'valuer l'intensit relative d'une source de lumire monochromatique en incluant un facteur d'interfrence
   * entre les diffrents oscillateurs composant la source de lumire. 
   * </p>
   * 
   * <p>
   * Les interprtations des intensits sont les suivantes :
   * <ul> I = 1     : Intensit avec <b>interfrence constructive</b>.</ul>
   * <ul> I = 0.5   : Intensit <b>sans interfrence</b> (intensit totale des oscillateurs sans interfrence).</ul>
   * <ul> I = 0     : Intensit avec <b>interfrence destructive</b>.</ul>
   * <ul> I = autre : Intensit avec <b>interfrence partielle</b>.</ul>
   *  </p>
   *  
   * @param position_to_illuminate La position o l'intensit sera value.
   * @return L'intensit relative de la source de lumire monochromatique.
   */
  public double getRelativeIntensity(SVector3d position_to_illuminate);
  
}//fin de l'interface SInterferenceLight
