/**
 * 
 */
package sim.graphics.shader;

import sim.exception.SRuntimeException;
import sim.geometry.SRay;
import sim.graphics.SColor;
import sim.math.SVector3d;

/**
 * Interface permet de dterminer la couleur associe  l'intersection d'un rayon avec une gomtrie en fonction du matriel appliqu sur la gomtrie, de l'clairage de la scne et des primitives environnant (ex : effet d'ombrage).
 * Il existe une trs grande varit de "shader" dont leur complexit amliore le ralisme de la couleur calcule.
 * 
 * @author Simon Vzina
 * @since 2015-01-09
 * @version 2015-08-14
 */
public interface SShader {

	/**
	 * Mthode qui dtermine la couleur associe  un rayon ayant effectu une intersection avec une gomtrie de la scne.
	 * Si le <b>rayon a touch une gomtrie</b>, un calcul sera ralis pour <b>dterminer la couleur</b> en lien avec le <b>point d'intersection</b>.
	 * Si le <b>rayon n'a pas touch de gomtrie</b>, un <b>test d'intersection sera ralis</b> en premier temps pour dterminer la gomtrie intersect et un calcul de couleur sera ralis par la suite.
	 * Une couleur </b>noire</b> sera retourne s'il n'y a <b>pas d'intersection</b> entre le rayon et une gomtrie de la scne.
	 * Il est important de rappeler que chaque gomtrie possde un lien vers sa primitive parent qui dtient l'information du matriel (ex: couleur de la gomtrie).
	 * @param ray - Le rayon ayant ralis une intersection.
	 * @return La couleur associe  l'intersection. S'il n'y a <b>pas d'intersection</b>, la couleur retourne sera <b>noire</b>.
	 * @throws SRuntimeException Si le rayon a dj t intersect palablement.
	 */
	public SColor shade(SRay ray)throws SRuntimeException;
	
	/**
   * Mthode pour valuer l'indice de rfraction associ  un point de l'espace.
   * Nous avons trois scnarios possibles :
   * <p>1) Si la position est situe  la <b>frontire d'une gomtrie</b> (sur la surface), elle sera considre comme  l'extrieure de la gomtrie et l'indice n = 1.0 sera affect.</p>
   * <p>2) Si la position est situe dans <b>une gomtrie</b>, il faudra obtenir son indice de rfraction n s'il possde une primitive comme parent, sinon n = 1.0 sera affect.</p>
   * <p>3) Si la position est situe dans <b>plusieurs gomtrie</b>, NOUS AVONS PRSENTEMENT UN PROBLME !!! qui devra tre rsolu dans le futur.</p>
   * @param position - La position dans l'espace des gomtries.
   * @return L'indice de rfraction associ  la position.
   */
  public double evaluateRefractiveIndex(SVector3d position);
  
}//fin interface SShader
